#include <acado_optimal_control.hpp>
#include <include/acado_gnuplot/gnuplot_window.hpp>
#include <acado/function/c_function.hpp>
#include <acado/curve/curve.hpp>

USING_NAMESPACE_ACADO

void differential_eval( double *xin, double *f, void *userData )
{
    double xt = xin[0];
    double xx = xin[1];
    //double l = xin[2];
    double xz = xin[3];
    double xu = xin[4];    
    double p = xin[5];    
    double xbc;
    ((ACADO::Curve*)userData)->evaluate(xt,&xbc);

    f[0] = -xx + 0.5*xx*xx + xu + 0.5*xz + xbc;  //x1'
    f[1] =  xx*xx + 3.0*xu*xu          ;  //x2'
    f[2] =  xz + exp(xz) - 1.0 + xx;     //==0
} 

int main( ){
   
// DEFINE A VARIABLES GRID:
    // ------------------------
    Grid dataGrid( 0.0, 10.0, 6 );

    VariablesGrid data;
    data.init( 1, dataGrid );

    data( 0, 0 ) = 1.0  ;
    data( 1, 0 ) = 0.8  ;
    data( 2, 0 ) = 0.7  ;
    data( 3, 0 ) = 0.65 ;
    data( 4, 0 ) = 0.625;
    data( 5, 0 ) = 0.613;

    // CONSTRUCT A CURVE INTERPOLATING THE DATA:
    // -----------------------------------------

    Curve c1;
    c1.add( data, IM_LINEAR   );//c1.add( data, IM_CONSTANT );

    // ----------------------------------------- 

    const double t_start =  0.0;
    const double t_end   = 10.0;
    
    // INTRODUCE THE VARIABLES:
    // -------------------------    
    DifferentialEquation      f;
    Parameter                 p;             // definition of a parameter   
    TIME                   tim;       
    DifferentialState x(2);
    AlgebraicState z;
    Control u;
    IntermediateState Naming(6); 
    Naming(0) = tim;                 
    Naming(1) = x(0);               
    Naming(2) = x(1); 
    Naming(3) = z; 
    Naming(4) = u;   
    Naming(5) = p;        
    
    // DEFINE A DIFFERENTIAL EQUATION:
    // -------------------------------

    CFunction cLinkModel_1( 3, differential_eval ); 
    cLinkModel_1.setUserData(&c1);    
    f << cLinkModel_1(Naming);    
            
    /*f << dot(x) == -x + 0.5*x*x + u + 0.5*z;
    f << dot(l) ==  x*x + 3.0*u*u + 0.001*t + 0.00000001*cmap(t) ;
    f <<      0 ==  z + exp(z) - 1.0 + x     ;
    f <<      0 ==  syste - t + 0.00000001*cmap(t) ;*/


    // DEFINE AN OPTIMAL CONTROL PROBLEM:
    // ----------------------------------
    OCP ocp( t_start, t_end, 100 );
    ocp.minimizeMayerTerm( x(1) );

    ocp.subjectTo( f );
    ocp.subjectTo( AT_START, x(0) == 1.0 );
    ocp.subjectTo( AT_START, x(1) == 0.0 );
    
    //ocp.subjectTo( -1.0 <= x*u <= 1.0 );     // a path constraint

    GnuplotWindow window;
        window.addSubplot(x(0),"DIFFERENTIAL STATE  x");
        window.addSubplot(z,"ALGEBRAIC STATE  z"   );
        window.addSubplot(u,"CONTROL u"            );
       // window.addSubplot(c1,t_start,t_end,"BC"            );      


    // DEFINE AN OPTIMIZATION ALGORITHM AND SOLVE THE OCP:
    // ----------------------------------------------------
    OptimizationAlgorithm algorithm(ocp); 
    
    algorithm.set( KKT_TOLERANCE, 1.000000E-05 );
    algorithm.set( RELAXATION_PARAMETER, 1.500000E+00 );        

	//algorithm.set( ABSOLUTE_TOLERANCE   , 1.0e-7        );
	//algorithm.set( INTEGRATOR_TOLERANCE , 1.0e-7        );
	//algorithm.set( HESSIAN_APPROXIMATION, EXACT_HESSIAN );
	//algorithm.set( GLOBALIZATION_STRATEGY, GS_FULLSTEP );

	algorithm << window;
    algorithm.solve();

    return 0;
}   



