#include <acado_optimal_control.hpp>
#include <include/acado_gnuplot/gnuplot_window.hpp>
#include <acado/function/c_function.hpp>
#include <acado/curve/curve.hpp>

//#define MAIN_CASETWODIM 
//#define MAIN_CASETWODIMQUADRATIC 
#define MAIN_CASE1D 

USING_NAMESPACE_ACADO


const double cSellPower=100.0;
const double cBuyPower=200.0;
const double cSellFinalPower=150.0;//in order for algorithm not to try to sell all the power before final time.
const double cSigmaBattDis=0.01;//battery discharge
const double cBattEfficiency=0.9;     //now same for charge and discharge. TODO - change.
const double cH2Efficiency=0.85;

const double cBattSigma=0.001;

const double t_start =  0.0;
    const double t_end   = 100.0;//82800.0;//seconds
    
    const double Wbattmax = 200000;//JOULES in battery
    const double Wfcmax = 10000000;//JOULES in hydrogen
    const double Wbattinit = Wbattmax/2.0;  //initial charge in battery
    
    const double Pbattplusmax = 1000; // joules per second maximal current of power to battery
    const double Pbattminusmax = -2000; // joules per second maximal current of power FROM battery
    const double Ph2plusmax = 900; // joules per second maximal current of power to hydrogen
    const double Ph2minusmax = -20000; // joules per second maximal current of power FROM hydrogen
    
void load_bc(Curve *c1,VariablesGrid *data)
{
// DEFINE A VARIABLES GRID:
    // ------------------------
                             
    //VariablesGrid data( "D:\\Data\\Projects\\optimalcontrol\\ACADOtoolkit\\examples\\my_examples\\data.txt" );
    
    /*Grid dataGrid( t_start, t_end, 6 );

    VariablesGrid data;
    data.init( 1, dataGrid );     //todo - nacist ze souboru.

    data( 0, 0 ) = 100*(0.0-0.5)  ;
    data( 1, 0 ) = 100*(0.2-0.5)  ;
    data( 2, 0 ) = 100*(0.8-0.5)  ;
    data( 3, 0 ) = 100*(0.9-0.5) ;
    data( 4, 0 ) = 100*(0.5-0.5);
    data( 5, 0 ) = 100*(0.0-0.5);  */
    
    Grid dataGrid( t_start, t_end, 21 );

    data->init( 1, dataGrid );     //todo - nacist ze souboru.

    (*data)( 0, 0 ) = -100  ;
    (*data)( 1, 0 ) = -100;
    (*data)( 2, 0 ) = -100;
    (*data)( 3, 0 ) = -500 ;
    (*data)( 4, 0 ) = -500;
    (*data)( 5, 0 ) = -500;
    (*data)( 6, 0 ) = -500;
    (*data)( 7, 0 ) = 348;
    (*data)( 8, 0 ) = 1449;
    (*data)( 9, 0 ) = 1494;
    (*data)( 10, 0 ) = 2427;
    (*data)( 11, 0 ) = 2722;
    (*data)( 12, 0 ) = 2643;
    (*data)( 13, 0 ) = 823;
    (*data)( 14, 0 ) = -500;
    (*data)( 15, 0 ) = -500;
    (*data)( 16, 0 ) = -500;
    (*data)( 17, 0 ) = -100;
    (*data)( 18, 0 ) = -100;
    (*data)( 19, 0 ) = -100;
    (*data)( 20, 0 ) = -100;

    // CONSTRUCT A CURVE INTERPOLATING THE DATA:
    // -----------------------------------------

    c1->add( (*data), IM_LINEAR   );//c1.add( data, IM_CONSTANT );
}

#ifdef MAIN_CASETWODIMQUADRATIC

void differential_eval( double *xin, double *f, void *userData )
{
    double t = xin[0];
    double x0 = xin[1];
    double x1 = xin[2];    
    double ubatt = xin[4];    
    double ucell = xin[5];    
    double bc;
    ((ACADO::Curve*)userData)->evaluate(t,&bc);

    if (ubatt>=0) //into batt
      f[0] = ubatt*cBattEfficiency - cBattSigma*x0;  //x1' ; for battery
    else
      f[0] = ubatt - cBattSigma*x0;
    if (ucell>=0) //into cell
      f[1] = ucell*cH2Efficiency;  //x2' ; for hydrogen
    else
      f[1] = ucell;
      
    double xPWO=bc-ubatt-ucell; //power out
    f[2]=xPWO*xPWO;
      
}




int main( ){
    
    VariablesGrid data;
    Curve c1;
    load_bc(&c1,&data); 

    // ----------------------------------------- 
    
    // INTRODUCE THE VARIABLES:
    // -------------------------    
    DifferentialEquation      f;
    //Parameter                 p;             // definition of a parameter   
    TIME                   tim;       
    DifferentialState x(2);//stored energy in: battery, cell
    DifferentialState SquarePower;//the accumulated cost //AlgebraicState?  
    Control UBatt;
    Control UH2;    

    IntermediateState Naming(6); 
    Naming(0) = tim;                 
    Naming(1) = x(0);               
    Naming(2) = x(1); 
    Naming(3) = SquarePower;
    Naming(4) = UBatt;   
    Naming(5) = UH2; 


    CFunction cLinkModel_1( 3, differential_eval );
    cLinkModel_1.setUserData(&c1);
    f << cLinkModel_1(Naming);   
    

    OCP ocp(t_start, t_end, 100);
    ocp.minimizeMayerTerm(SquarePower);
    ocp.subjectTo(f);
    
    ocp.subjectTo( AT_START, x(0) == Wbattinit );
    ocp.subjectTo( AT_START, x(1) == 0.0 ); 
    
    ocp.subjectTo( Pbattminusmax <= UBatt <= Pbattplusmax );     // a path constraints Ubatt
    ocp.subjectTo( Ph2minusmax <= UH2 <= Ph2plusmax );     // a path constraints Ucell
    ocp.subjectTo( 0.0 <= x(0) <= Wbattmax );   
    ocp.subjectTo( 0.0 <= x(1) <= Wfcmax );

    GnuplotWindow window;
        window.addSubplot(x(0),"Battery Stored work");
        window.addSubplot(x(1),"Hydrogen Stored work");
        window.addSubplot(UBatt,"Control - battery");
        window.addSubplot(UH2,"Control - fuellcell");
        window.addSubplot(c1,t_start,t_end,"Power demand conditions");


    OptimizationAlgorithm algo1(ocp);
    //algo1.set( KKT_TOLERANCE, 1.000000E-05 );
    algo1.set( RELAXATION_PARAMETER, 1.500000E+00 );
    algo1.set(MAX_NUM_ITERATIONS, 10000);

 // algo1.set( HESSIAN_APPROXIMATION, EXACT_HESSIAN ); s tutim to nefunguje.
 // 
    algo1 << window;
    
    //algo1.initializeDifferentialStates( "states.txt" );  
    //algo1.initializeControls          ( "controls.txt" );
    //algo1.initializeParameters        ( "parameters.txt" );

    algo1.solve();
    
    algo1.getDifferentialStates("states.txt"    );
    algo1.getParameters        ("parameters.txt");
    algo1.getControls          ("controls.txt"  );

    return 0;
}  
#endif

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------  

#ifdef MAIN_CASETWODIM  

void differential_eval_q( double *xin, double *f, void *userData )
{
   double t = xin[0];
    double x0 = xin[1];
    double x1 = xin[2];
    //double soldPower = xin[3];    
    double ubatt = xin[4];    
    double ucell = xin[5];    
    double bc;
    ((ACADO::Curve*)userData)->evaluate(t,&bc);

    if (ubatt>=0) //into batt
      f[0] = ubatt*cBattEfficiency - cBattSigma*x0;  //x1' ; for battery
    else
      f[0] = ubatt - cBattSigma*x0;
    if (ucell>=0) //into cell
      f[1] = ucell*cH2Efficiency;  //x2' ; for hydrogen
    else
      f[1] = ucell;
      
    double xPWO=bc-ubatt-ucell; //power out
    if (xPWO>=0.0) // output power to grid - selling the electricity
    {
      f[2] = xPWO*cSellPower;     //sold'
    }
    else //buy input power from grid
    {
      f[2] = xPWO*cBuyPower;     //bought'
    }      
}




int main( ){
    
    VariablesGrid data;
    Curve c1;
    load_bc(&c1,&data);  

    // ----------------------------------------- 
    
    // INTRODUCE THE VARIABLES:
    // -------------------------    
    DifferentialEquation      f;
    //Parameter                 p;             // definition of a parameter   
    TIME                   tim;       
    DifferentialState x(2);//stored energy in: battery, cell
    DifferentialState CostPower;//the accumulated cost //AlgebraicState?    
    Control UBatt;
    Control UH2;    
    IntermediateState Naming(6); 
    Naming(0) = tim;                 
    Naming(1) = x(0);               
    Naming(2) = x(1); 
    Naming(3) = CostPower;     
    Naming(4) = UBatt;   
    Naming(5) = UH2;

    CFunction cLinkModel_1( 3, differential_eval_q );
    cLinkModel_1.setUserData(&c1);
    f << cLinkModel_1(Naming);   
    

    OCP ocp(t_start, t_end, 2);
    ocp.maximizeMayerTerm(CostPower+(x(0)+x(1))*cSellFinalPower);
    ocp.subjectTo(f);
    
    ocp.subjectTo( AT_START, x(0) == Wbattinit );
    ocp.subjectTo( AT_START, x(1) == 0.0 );
    ocp.subjectTo( AT_START, CostPower == 0.0 ); 
    
    ocp.subjectTo( Pbattminusmax <= UBatt <= Pbattplusmax );     // a path constraints Ubatt
    ocp.subjectTo( Ph2minusmax <= UH2 <= Ph2plusmax );     // a path constraints Ucell
    ocp.subjectTo( 0.0 <= x(0) <= Wbattmax );   
    ocp.subjectTo( 0.0 <= x(1) <= Wfcmax );     


    GnuplotWindow window;
        window.addSubplot(x(0),"Battery Stored work");
        window.addSubplot(x(1),"Hydrogen Stored work");
        window.addSubplot(CostPower,"CostPower");
        window.addSubplot(UBatt,"Control - battery");
        window.addSubplot(UH2,"Control - fuellcell");
        window.addSubplot(c1,t_start,t_end,"Power demand conditions");


    OptimizationAlgorithm algo1(ocp);
    //algo1.set( KKT_TOLERANCE, 1.000000E-05 );
    algo1.set( RELAXATION_PARAMETER, 1.500000E+00 );
    algo1.set(MAX_NUM_ITERATIONS, 10000);

 // algo1.set( HESSIAN_APPROXIMATION, EXACT_HESSIAN ); s tutim to nefunguje.
 // 
    algo1 << window;
    
    algo1.initializeDifferentialStates( "states.txt" );  
    algo1.initializeControls          ( "controls.txt" );
    algo1.initializeParameters        ( "parameters.txt" );

    algo1.solve();
    
    algo1.getDifferentialStates("states.txt"    );
    algo1.getParameters        ("parameters.txt");
    algo1.getControls          ("controls.txt"  );

    return 0;
}
#endif
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#ifdef MAIN_CASE1D 
void differential_eval_d( double *xin, double *f, void *userData )
{
    double t = xin[0];
    double x0 = xin[1];
    double x1 = xin[2];    
        
    double ubatt = xin[5];                  //smer z baterii
    double bc;
    ((ACADO::Curve*)userData)->evaluate(t,&bc);
    double ufc=bc-ubatt;//zbytek bereme z fcell
    double xout=0;
    if (ufc>0)
    {
      if (x1<=0) //ale v fcell uz nic neni
      {
        xout=-ufc*cBuyPower;//kupujeme elektrinu a ztracime penize
        ufc=0;
      }
      else if (ufc>Ph2plusmax)
      {
        ufc=Ph2plusmax;
        xout=-(bc-ubatt-ufc)*cBuyPower;//kupujeme elektrinu a ztracime penize
        
      }
    }
    else      //nabijime fuelcell
    {
      if (x1>=Wfcmax)//vic to nejde
      {
        xout=-ufc*cSellPower;//a prodavame elektrinu
        ufc=0;
      }
      else if (ufc<Ph2minusmax)
      {
        ufc=-Ph2minusmax;
        xout=-(bc-ubatt-ufc)*cSellPower;//kupujeme elektrinu a ztracime penize      
      }
    }

    if (ubatt>=0) //into batt
      f[0] = ubatt*cBattEfficiency - cBattSigma*x0;  //x1' ; for battery
    else
      f[0] = ubatt - cBattSigma*x0;
    if (ufc>=0) //into cell
      f[1] = ufc*cH2Efficiency;  //x2' ; for hydrogen
    else
      f[1] = ufc;
      
    f[2]=xout;      
}




int main( ){
    
    VariablesGrid data;
    Curve c1;
    load_bc(&c1,&data);  

    // ----------------------------------------- 
    
    // INTRODUCE THE VARIABLES:
    // -------------------------    
    DifferentialEquation      f;
    //Parameter                 p;             // definition of a parameter   
    TIME                   tim;       
    DifferentialState x(2);//stored energy in: battery, cell
    DifferentialState PowerCost;//the accumulated cost    
    Control UBatt;

    IntermediateState Naming(5); 
    Naming(0) = tim;                 
    Naming(1) = x(0);               
    Naming(2) = x(1); 
    Naming(3) = PowerCost;    
    Naming(4) = UBatt;   
  


    CFunction cLinkModel_1( 3, differential_eval_d );
    cLinkModel_1.setUserData(&c1);
    f << cLinkModel_1(Naming);   
    

    OCP ocp(t_start, t_end, 10);
    ocp.maximizeMayerTerm(PowerCost+(x(0)+x(1))*cSellFinalPower);
    ocp.subjectTo(f);
    
    ocp.subjectTo( AT_START, x(0) == Wbattinit );
    ocp.subjectTo( AT_START, x(1) == 0.0 );
    ocp.subjectTo( AT_START, PowerCost == 0.0 ); 
    
    ocp.subjectTo( Pbattminusmax <= UBatt <= Pbattplusmax );     // a path constraints Ubatt
    //ocp.subjectTo( Ph2minusmax <= UH2 <= Ph2plusmax );     // a path constraints Ucell
    ocp.subjectTo( 0.0 <= x(0) <= Wbattmax );   
    ocp.subjectTo( 0.0 <= x(1) <= Wfcmax );

    GnuplotWindow window;
        window.addSubplot(x(0),"Battery Stored work");
        window.addSubplot(x(1),"Hydrogen Stored work");
        window.addSubplot(PowerCost,"PowerCost");
        window.addSubplot(UBatt,"Control - battery");
        //window.addSubplot(UH2,"Control - fuellcell");
        window.addSubplot(c1,t_start,t_end,"Power demand conditions");


    OptimizationAlgorithm algo1(ocp);
    //algo1.set( KKT_TOLERANCE, 1.000000E-05 );
    algo1.set( RELAXATION_PARAMETER, 1.500000E+00 );
    algo1.set(MAX_NUM_ITERATIONS, 10000);

 // algo1.set( HESSIAN_APPROXIMATION, EXACT_HESSIAN ); s tutim to nefunguje.
 // 
    algo1 << window;
    
    //algo1.initializeDifferentialStates( "states.txt" );  
    //algo1.initializeControls          ( "controls.txt" );
    //algo1.initializeParameters        ( "parameters.txt" );
    
    algo1.initializeControls(data);

    algo1.solve();
    
    algo1.getDifferentialStates("states.txt"    );
    algo1.getParameters        ("parameters.txt");
    algo1.getControls          ("controls.txt"  );

    return 0;
} 
#endif 

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------  


 /*
void differential_eval( double *xin, double *f, void *userData )
{
    double t = xin[0];
    double x1 = xin[1];
    double x2 = xin[2];
    double cost = xin[3];
    double ubatt = xin[4];    
    double ucell = xin[5];    
    double bc;
    ((ACADO::Curve*)userData)->evaluate(t,&bc);

    f[0] = ubatt*cBattEfficiency;  //x1' ; for battery
    f[1] = ucell*cH2Efficiency;  //x2' ; for hydrogen
    double xPWO=bc-ubatt-ucell; //power out
    if (xPWO>=0.0) // output power to grid - selling the electricity
      f[2] = cost-xPWO*cSellPower;     //cost'
    else //buy input power from grid
      f[2] = cost-xPWO*cBuyPower;     //cost' 
} 

int main( ){        
    
// DEFINE A VARIABLES GRID:
    // ------------------------
    Grid dataGrid( t_start, t_end, 6 );

    VariablesGrid data;
    data.init( 1, dataGrid );     //todo - nacist ze souboru.

    data( 0, 0 ) = 0.0  ;
    data( 1, 0 ) = 0.2  ;
    data( 2, 0 ) = 0.8  ;
    data( 3, 0 ) = 0.9 ;
    data( 4, 0 ) = 0.5;
    data( 5, 0 ) = 0.0;

    // CONSTRUCT A CURVE INTERPOLATING THE DATA:
    // -----------------------------------------

    Curve c1;
    c1.add( data, IM_LINEAR   );//c1.add( data, IM_CONSTANT ); 

    // ----------------------------------------- 
    
    // INTRODUCE THE VARIABLES:
    // -------------------------    
    DifferentialEquation      f;
    //Parameter                 p;             // definition of a parameter   
    TIME                   tim;       
    DifferentialState x(2);//stored energy in: battery, cell
    DifferentialState Cost;//the accumulated cost //AlgebraicState?
    Control UBatt;
    Control UH2;    
    IntermediateState Naming(6); 
    Naming(0) = tim;                 
    Naming(1) = x(0);               
    Naming(2) = x(1); 
    Naming(3) = Cost; 
    Naming(4) = UBatt;   
    Naming(5) = UH2;        
    
    // DEFINE A DIFFERENTIAL EQUATION:
    // -------------------------------

    CFunction cLinkModel_1( 3, differential_eval ); 
    cLinkModel_1.setUserData(&c1);    
    f << cLinkModel_1(Naming);    

    // DEFINE AN OPTIMAL CONTROL PROBLEM:
    // ----------------------------------
    OCP ocp( t_start, t_end, 10 );
    ocp.maximizeMayerTerm( Cost );
    //ocp.maximizeLagrangeTerm( Cost+x(0)*cSellFinalPower+x(1)*cSellFinalPower );
    ocp.minimizeLagrangeTerm( x);

    ocp.subjectTo( f );
    ocp.subjectTo( AT_START, x(0) == Wbattinit );
    ocp.subjectTo( AT_START, x(1) == 0.0 );
    ocp.subjectTo( AT_START, Cost == 0.0 ); 
    
    ocp.subjectTo( Pbattminusmax <= UBatt <= Pbattplusmax );     // a path constraints Ubatt
    ocp.subjectTo( Ph2minusmax <= UH2 <= Ph2plusmax );     // a path constraints Ucell
    ocp.subjectTo( 0.0 <= x(0) <= Wbattmax );     // a path constraints:
    ocp.subjectTo( 0.0 <= x(1) <= Wfcmax );     // a path constraints:       
    
    GnuplotWindow window;
        window.addSubplot(x(0),"Battery Stored work");
        window.addSubplot(x(1),"Hydrogen Stored work");
        window.addSubplot(Cost,"Cost");
        window.addSubplot(UBatt,"Control - battery");
        window.addSubplot(UH2,"Control - fuellcell");
        window.addSubplot(c1,t_start,t_end,"Power demand conditions");       


    // DEFINE AN OPTIMIZATION ALGORITHM AND SOLVE THE OCP:
    // ----------------------------------------------------
    OptimizationAlgorithm algorithm(ocp); 
    
    algorithm.set( KKT_TOLERANCE, 1.000000E-05 );
    algorithm.set( RELAXATION_PARAMETER, 1.500000E+00 );        

	//algorithm.set( ABSOLUTE_TOLERANCE   , 1.0e-7        );
	//algorithm.set( INTEGRATOR_TOLERANCE , 1.0e-7        );
	//algorithm.set( HESSIAN_APPROXIMATION, EXACT_HESSIAN );
	//algorithm.set( GLOBALIZATION_STRATEGY, GS_FULLSTEP );

	  algorithm << window;
    algorithm.solve();

    return 0;
}     */  
