clear all;
clf;
format long;
consts;

%
% MULTIPLE SHOOTING
%

 % Options for NLP Solvers
 optODE = odeset( 'RelTol', 1e-8, 'AbsTol', 1e-8 );
%'GradObj','on','DerivativeCheck','on','HessUpdate','bfgs','Diagnostic','on','LineSearchType','cubicpoly',
%GradConstr on
 % Time Horizon and Initial State
 %Data.t_start; % Initial time
 %Data.t_end; % Final time

x0 = [ Data.Wbattinit; 0; 0 ]; % Initial states+1 state for cost variable

ndisc = 2^(Data.cLogBegin-1);
 
w0 = zeros(Data.numcontrols*ndisc+Data.numstates*(ndisc-1),1);%and each control has startvalues (one less than controls) - according to multiple shooting
for is = Data.cLogBegin:Data.cLogEnd
ndisc = 2*ndisc; % Number of stages: ns = 2, 4, 8, 16, and 32
ts = [Data.t_start:(Data.t_end-Data.t_start)/ndisc:Data.t_end]; % Time stages (equipartition)

% Initial Guess and Bounds for the Parameters
wold=w0;
w0 = zeros(Data.numcontrols*ndisc+Data.numstates*(ndisc-1),1);
for (inumcontrols=1:(Data.numcontrols))
   for (jsi=1:ndisc)%use previous results as initial guess.
        w0(jsi+(inumcontrols-1)*Data.numcontrols,1) = wold(1+floor((jsi-1)/2)+(inumcontrols-1)*(ndisc/2),1);
   end
end 
for (inumstates=1:(Data.numstates))
   for (jsi=2:(ndisc-1))%ssame for initial points
        w0(ndisc*Data.numcontrols+jsi+(ndisc-1)*(inumstates-1),1) = ...
            wold(1+floor((jsi-1-1)/2)+(ndisc/2)*Data.numcontrols+(ndisc/2-1)*(inumstates-1),1);
   end
   w0(ndisc*Data.numcontrols+1+(ndisc-1)*(inumstates-1),1) = 0.0;%new appeared at the begining - set to initial value .. or zero it doesnot matter that much
end    
 
%bounds for control variables: (conrolScales are multiplied in the equation so here we must divide)
%controls: battin,battou,h2in,h2out
%means:    pbattplus,pbattminus,ph2plus,ph2minus
wL = zeros(Data.numcontrols*ndisc+Data.numstates*(ndisc-1),1); %parameter (=control) bounds in each tie stage
wU = Data.Pbattplusmax/Data.ControlScales*ones(Data.numcontrols*ndisc+Data.numstates*(ndisc-1),1);
wU((ndisc+1):(2*ndisc),1)=abs(Data.Pbattminusmax)/Data.ControlScales*ones(ndisc,1);
wU((2*ndisc+1):(3*ndisc),1)=Data.Ph2plusmax/Data.ControlScales*ones(ndisc,1); 
wU((3*ndisc+1):(4*ndisc),1)=abs(Data.Ph2minusmax)/Data.ControlScales*ones(ndisc,1); 
%bounds for states - on multipleshooting it equals bounds for control
%variables:
 %bounds on the beginnings are just bounds on state variables:
wL((4*ndisc+1):(4*ndisc+(ndisc-1)),1)=0*zeros((4*ndisc+(ndisc-1)-(4*ndisc+1)+1),1);
wU((4*ndisc+1):(4*ndisc+(ndisc-1)),1)=Data.Wbattmax*ones((4*ndisc+(ndisc-1)-(4*ndisc+1)+1),1);
wL((4*ndisc+(ndisc-1)+1):(4*ndisc+2*(ndisc-1)),1)=0*zeros((4*ndisc+2*(ndisc-1))-(4*ndisc+(ndisc-1)+1)+1,1);
wU((4*ndisc+(ndisc-1)+1):(4*ndisc+2*(ndisc-1)),1)=Data.Wfcmax*ones((4*ndisc+2*(ndisc-1))-(4*ndisc+(ndisc-1)+1)+1,1);
 
 

%OPTI toolbox:
 
 % Options
opts = optiset('solver','ipopt','display','iter','maxtime',1000*60*60);
[ret,c,ceq]=ctr(x0,ts,w0,Data);%test run to see the dimensions of control problems equalities and inequalities
sizecr=size(c);%row vectors, so the size is the second element
sizeceqr=size(ceq);
sizec=sizecr(2);
sizeceq=sizeceqr(2);

cl = [-Inf*ones(1,sizec),zeros(1,sizeceq)];
cu = [zeros(1,sizec),zeros(1,sizeceq)];

% Build OPTI Problem
Opt = opti('fun',@(ws)obj(x0,ts,ws,Data),'nl',@(ws)ctr(x0,ts,ws,Data),cl,cu,'bounds',wL,wU,'options',opts)

% Solve NLP
[wopt,fval,exitflag,info] = solve(Opt,w0)

dispopt( x0, ts, wopt,Data, optODE,-1 );
dlmwrite('controldata',wopt);



%{
optNLP = optimset('Algorithm','sqp', 'LargeScale', 'off', 'GradObj', 'off', 'GradConstr', 'off',...
'DerivativeCheck', 'off', 'Display', 'iter-detailed', 'TolX', 1e-9,...
'TolFun', 1e-9, 'TolCon', 1e-9, 'MaxFunEval', 8000,...
'DiffMinChange', 1e-5,...
'OutputFcn',@(x,optimValues,state)nlpoutfun(x0,x,optimValues,state, ts, Data, optODE));%'PlotFcns'

% Sequential Approach of Dynamic Optimization
[ wopt ] = fmincon( @(ws)obj(x0,ts,ws,Data), w0, [], [], [], [], wL, wU,...
@(ws)ctr(x0,ts,ws,Data), optNLP);
dispopt( x0, ts, wopt,Data, optODE,-1 );
dlmwrite('controldata',wopt);
%}
end