clear all;
clf;
format long;
consts;

%
% MULTIPLE SHOOTING
%

 % Options for NLP Solvers
 optODE = odeset( 'RelTol', 1e-8, 'AbsTol', 1e-8 );
 optNLP = optimset( 'LargeScale', 'off', 'GradObj', 'off', 'GradConstr', 'off',...
 'DerivativeCheck', 'off', 'Display', 'iter', 'TolX', 1e-9,...
 'TolFun', 1e-9, 'TolCon', 1e-9, 'MaxFunEval', 600,...
 'DiffMinChange', 1e-5 );
%'GradObj','on','DerivativeCheck','on','HessUpdate','bfgs','Diagnostic','on','LineSearchType','cubicpoly',
%GradConstr on
 % Time Horizon and Initial State
 %Data.t_start; % Initial time
 %Data.t_end; % Final time
 
 % Time Horizon and Initial State
Data.t_start = 0; % Initial time
Data.t_end = 2; % Final time
x0 = [ 1; 1; 0 ]; % Initial state
 


 ns = 2^(Data.cLogBegin-1);
 
 w0 = zeros(Data.numcontrols*ns+Data.numstates*(ns-1),1);%and each control has startvalues (one less than controls) - according to multiple shooting
 for is = Data.cLogBegin:Data.cLogEnd
 ns = 2*ns; % Number of stages: ns = 2, 4, 8, 16, and 32
 ts = [Data.t_start:(Data.t_end-Data.t_start)/ns:Data.t_end]; % Time stages (equipartition)

 % Initial Guess and Bounds for the Parameters
 wold=w0;
 w0 = zeros(Data.numcontrols*ns+Data.numstates*(ns-1),1);
 for (inumcontrols=1:(Data.numcontrols))
    for (jsi=1:ns)%use previous results as initial guess.
         w0(jsi+(inumcontrols-1)*Data.numcontrols,1) = wold(1+floor((jsi-1)/2)+(inumcontrols-1)*(ns/2),1);
    end
 end 
 for (inumstates=1:(Data.numstates))
    for (jsi=2:(ns-1))%ssame for initial points
         w0(ns*Data.numcontrols+jsi+(ns-1)*inumstates,1) = ...
             wold(1+floor((jsi-1)/2)+(ns/2)*Data.numcontrols+(ns-1)*inumstates,1);
    end
    w0(ns*Data.numcontrols+1+(ns-1)*(inumstates-1),1) = 0.0;%new appeared at the begining - set to initial value .. or zero it doesnot matter that much
 end    
 
 wL = Data.Pbattminusmax*ones(Data.numcontrols*ns+Data.numstates*(ns-1),1); %parameter (=control) bounds in each tie stage
 wU = Data.Pbattplusmax*ones(Data.numcontrols*ns+Data.numstates*(ns-1),1);
 wL((ns+1):(2*ns),1)=Data.Ph2minusmax*ones(2*ns-(ns+1)+1,1);
 wU((ns+1):(2*ns),1)=Data.Ph2plusmax*ones(2*ns-(ns+1)+1,1); 
  %bounds on the beginnings are just bounds on state variables:
 wL((2*ns+1):(2*ns+(ns-1)),1)=0*zeros((2*ns+(ns-1)-(2*ns+1)+1),1);
 wU((2*ns+1):(2*ns+(ns-1)),1)=Data.Wbattmax*ones((2*ns+(ns-1)-(2*ns+1)+1),1);
 wL((2*ns+(ns-1)+1):(2*ns+2*(ns-1)),1)=0*zeros((2*ns+2*(ns-1))-(2*ns+(ns-1)+1)+1,1);
 wU((2*ns+(ns-1)+1):(2*ns+2*(ns-1)),1)=Data.Wfcmax*ones((2*ns+2*(ns-1))-(2*ns+(ns-1)+1)+1,1);
 
 Data.currentns=ns;

 % Sequential Approach of Dynamic Optimization
 [ wopt ] = fmincon( @(ws)obj(x0,ns,ts,ws,Data), w0, [], [], [], [], wL, wU,...
 @(ws)ctr(x0,ns,ts,ws,Data), optNLP);
 dispopt( x0, ns, ts, wopt,Data, optODE );
 dlmwrite('controldata',wopt);

end